package com.olimex.bmp085;

import com.olimex.libi2c.CommunicationException;
import com.olimex.libi2c.libI2C;

import android.content.Context;
import android.util.Log;
import android.widget.Toast;

public class BMP085 implements Runnable {

	Thread measureThread;
	
	protected int OSS = 1;

	Context context;

	static int[] in_buffer = new int[22];
	private int UT;
	private long UP;

	static boolean isConnected = false;

	protected double temperature;
	protected double pressure;
	protected double altitude;
	protected int AC1;
	protected int AC2;
	protected int AC3;
	protected int AC4;
	protected int AC5;
	protected int AC6;
	protected int B1;
	protected int B2;
	protected int MB;
	protected int MC;
	protected int MD;

	/**
	 * If this flag is set values cannot be calculated, because the variables
	 * are updating.
	 */
	public static boolean isMeasuring;

	/**
	 * Start the measure thread.
	 */
	public void start() {
		if (measureThread == null) {
			measureThread = new Thread(this);
			measureThread.start();
		}
	}

	/**
	 * Stop the measure thread.
	 */
	public void stop() {
		if (measureThread != null) {
			measureThread.interrupt();
		}
	}

	@Override
	public void run() {

		try {
			Log.i("MEASURE", "Thread starting.");
			while (!Thread.interrupted()) {
				try {
					/* Set measure flag */
					isMeasuring = true;

					/* Read uncompressed temperature value */
					libI2C.writeData(0xF4, new int[] { 0x2E });
					Thread.sleep(5);
					in_buffer = libI2C.readData(0xF6, 2);
					UT = in_buffer[0] << 8 | in_buffer[1];

					/* Read uncompressed pressure value */
					libI2C.writeData(0xF4,
							new int[] { 0x34 + (OSS << 6) });
					switch (OSS) {
					case 0:
						Thread.sleep(5);
						break;
					case 1:
						Thread.sleep(8);
						break;
					case 2:
						Thread.sleep(14);
						break;
					case 3:
						Thread.sleep(26);
						break;
					}
					in_buffer = libI2C.readData(0xF6, 3);
					UP = ((in_buffer[0] << 16) | (in_buffer[1] << 8) | in_buffer[2]) >> (8 - OSS);

					/* Calculate temperature */
					long x1 = (long) (((UT - AC6) * AC5) / Math.pow(2, 15));
					long x2 = (long) (MC * Math.pow(2, 11) / (x1 + MD));
					long b5 = x1 + x2;
					long T = (long) ((b5 + 8) / Math.pow(2, 4));

					temperature = T * 0.1;

					/* Calculate pressure */
					long b6 = b5 - 4000;
					x1 = (long) ((B2 * (b6 * b6 / Math.pow(2, 12))) / Math.pow(
							2, 11));
					x2 = (long) ((AC2 * b6) / Math.pow(2, 11));
					long x3 = x1 + x2;
					long b3 = (((AC1 * 4 + x3) << OSS) + 2) / 4;
					x1 = (long) (AC3 * b6 / Math.pow(2, 13));
					x2 = (long) (B1 * (b6 * b6 / Math.pow(2, 12)) / Math.pow(2,
							16));
					x3 = (long) (((x1 + x2) + 2) / Math.pow(2, 2));
					long b4 = (long) (AC4 * ((x3 + 32768) / Math.pow(2, 15)));
					long b7 = (UP - b3) * (50000 >> OSS);
					long p;
					if (b7 < 0x80000000)
						p = (b7 * 2) / b4;
					else
						p = (b7 / b4) * 2;
					x1 = (long) ((p / Math.pow(2, 8)) * (p / Math.pow(2, 8)));
					x1 = (long) ((x1 * 3038) / Math.pow(2, 16));
					x2 = (long) ((-7357 * p) / Math.pow(2, 16));
					p = (long) (p + (x1 + x2 + 3791) / Math.pow(2, 4));

					pressure = p * 0.01;

					/* Calculate altitude */

					double alt;
					alt = Math.pow(((p / 100) / 1013.25), (1 / 5.255));

					altitude = 44330 * (1 - alt);

				} catch (CommunicationException ex) {
					Log.e("Measure", ex.getMessage());
					this.stop();
				} finally {
					isMeasuring = false;
					Thread.sleep(100);
				}
			}
			Log.i("MEASURE", "Thread stopping.");
		} catch (InterruptedException ex) {
			Log.i("MEASURE",
					"Thread shutting down as it was requested to stop.");

		} finally {
			measureThread = null;
		}
	}

	/**
	 * Define constructors
	 */

	public BMP085() {
	}

	public BMP085(Context context) {
		this.context = context;
	}

	/**
	 * Connect to i2c-bus and make initialisation of the board
	 * 
	 * @return
	 */
	public boolean Connect() {
		try {
			getParameters();
			setParameters();
			isConnected = true;
			return true;
		} catch (CommunicationException ex) {
			Toast.makeText(context.getApplicationContext(), ex.getMessage(),
					Toast.LENGTH_LONG).show();
			return false;
		}

	}

	/**
	 * Disconnect from i2c-bus
	 */
	public void Disconnect() {
		libI2C.fd = -1;
		isConnected = false;
	}

	/**
	 * Convert read raw data from getParameters
	 */
	private void setParameters() {
		AC1 = unsignedToSigned(in_buffer[0] << 8 | in_buffer[1]);
		AC2 = unsignedToSigned(in_buffer[2] << 8 | in_buffer[3]);
		AC3 = unsignedToSigned(in_buffer[4] << 8 | in_buffer[5]);
		AC4 = in_buffer[6] << 8 | in_buffer[7];
		AC5 = in_buffer[8] << 8 | in_buffer[9];
		AC6 = in_buffer[10] << 8 | in_buffer[11];
		B1 = unsignedToSigned(in_buffer[12] << 8 | in_buffer[13]);
		B2 = unsignedToSigned(in_buffer[14] << 8 | in_buffer[15]);
		MB = unsignedToSigned(in_buffer[16] << 8 | in_buffer[17]);
		MC = unsignedToSigned(in_buffer[18] << 8 | in_buffer[19]);
		MD = unsignedToSigned(in_buffer[20] << 8 | in_buffer[21]);
	}

	/**
	 * Convert unsigned short to signed
	 * 
	 * @param value
	 *            Unsigned number to be converted
	 * @return 2's complement of value
	 */
	private int unsignedToSigned(int value) {
		if ((value & 0x8000) > 0)
			return (value - (0xFFFF + 1));
		else
			return value;
	}

	/**
	 * Read BMP085 stored constants needed for calculation of temperature,
	 * pressure and altitude
	 * 
	 * @throws CommunicationException
	 */
	private void getParameters() throws CommunicationException {

		try {
			in_buffer = libI2C.readData(REGISTERS.AC1_MSB.getAddress(), 22);
		} catch (CommunicationException ex) {
			throw ex;
		}
	}
	
	public double getTemperature(){
		return temperature;
	}
	
	public double getPressure(){
		return pressure;
	}
	
	public double getAltitude(){
		return altitude;
	}


}
